@@ -28,8 +28,10 @@
 #include "poi.h"
 #include "wp.h"
 #include "tracks.h"
+#include "route.h"
 #include "hrm_functions.h"
 
+/* How many move events must come in for a drag to be recognized: */
 #define WTFCOUNTER 5
 
 
@@ -77,6 +79,7 @@ set_cursor(int type)
 		gdk_window_set_cursor(window1->window, cursor_default);
 }
 
+waypoint_t *selected_wp;
 
 gboolean
 on_drawingarea1_button_press_event     (GtkWidget       *widget,
@@ -97,6 +100,8 @@ on_drawingarea1_button_press_event     (GtkWidget       *widget,
 	local_x = global_x;
 	local_y = global_y;
 		
+	selected_wp = find_routepoint (mouse_x, mouse_y);
+	
 	return FALSE;
 }
 
@@ -147,32 +152,39 @@ on_drawingarea1_button_release_event   (GtkWidget       *widget,
 		
 		if(wtfcounter >= WTFCOUNTER)
 		{
+			/* This is the mouse release event
+			   of a previous drag: */
 			printf("* mouse drag +8events\n");
-			int mouse_dx, mouse_dy;
-			
-			global_x = local_x;
-			global_y = local_y;
-			
-			mouse_dx = mouse_x - (int) event->x;
-			mouse_dy = mouse_y - (int) event->y;
-			
-			global_x += mouse_dx;
-			global_y += mouse_dy;
-		
-			gdk_draw_rectangle (
-				pixmap,
-				widget->style->white_gc,
-				TRUE,
-				0, 0,
-				widget->allocation.width+260,
-				widget->allocation.height+260);
-						
-			gtk_widget_queue_draw_area (
-				widget, 
-				0,0,widget->allocation.width+260,widget->allocation.height+260);
-			
-		
-			repaint_all();
+
+			if (!selected_wp) {
+				int mouse_dx, mouse_dy;
+
+				global_x = local_x;
+				global_y = local_y;
+
+				mouse_dx = mouse_x - (int) event->x;
+				mouse_dy = mouse_y - (int) event->y;
+
+				global_x += mouse_dx;
+				global_y += mouse_dy;
+
+				gdk_draw_rectangle (
+					pixmap,
+					widget->style->white_gc,
+					TRUE,
+					0, 0,
+					widget->allocation.width+260,
+					widget->allocation.height+260);
+
+				gtk_widget_queue_draw_area (
+					widget, 
+					0,0,widget->allocation.width+260,widget->allocation.height+260);
+
+
+				repaint_all ();
+			} else {
+				selected_wp = NULL;
+			}
 		}
 		
 		if (abs(mouse_x - (int) event->x) < 10 && abs(mouse_y - (int) event->y) < 10)
@@ -181,6 +193,7 @@ on_drawingarea1_button_release_event   (GtkWidget       *widget,
 			gboolean friend_found = FALSE;
 			gboolean photo_found = FALSE;
 			gboolean poi_found = FALSE;
+			waypoint_t *routepoint;
 			
 			
 			if(global_show_friends)
@@ -231,8 +244,9 @@ on_drawingarea1_button_release_event   (GtkWidget       *widget,
 				}
 			}
 			
+			routepoint = find_routepoint (mouse_x, mouse_y);
 			
-			if (!friend_found && !photo_found && !poi_found && 
+			if (!friend_found && !photo_found && !poi_found && !routepoint && 
 				!distance_mode && !pickpoint_mode)
 			{	
 	
@@ -255,6 +269,13 @@ on_drawingarea1_button_release_event   (GtkWidget       *widget,
 					on_item10_activate(NULL, NULL);
 				if (poi_found)
 					on_item15_activate(NULL, NULL);
+				if (routepoint) {
+					gtk_widget_show (route_menu);
+					gtk_menu_popup (GTK_MENU (route_menu),
+					                NULL, NULL, NULL, NULL, 
+					                event->button,
+					                event->time);
+				}
 			}
 		}
 	
@@ -291,6 +312,15 @@ on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
 	 
 		if (state & GDK_BUTTON1_MASK  && wtfcounter>=WTFCOUNTER) 
 		{
+
+			if (selected_wp) {
+				double lat = pixel2lat (global_zoom, global_y+event->y);
+				double lon = pixel2lon (global_zoom, global_x+event->x);
+				change_waypoint_of_route (selected_wp, lat, lon);
+				repaint_all ();
+				return FALSE;
+			}
+
 			GtkToggleToolButton *autocenter_toggle;
 
 			if(!drag_started)
@@ -2248,12 +2278,13 @@ repaint_all()
 
 
 	fill_tiles_pixel(global_x, global_y, global_zoom, FALSE);
-	print_track();
+	paint_track();
 	paint_loaded_track();
 	paint_friends();
 	paint_photos();
 	paint_pois();
 	paint_wp();
+	paint_route();
 	paint_myposition();
 	
 }
@@ -2408,15 +2439,15 @@ on_drawingarea1_key_press_event        (GtkWidget       *widget,
                                         GdkEventKey     *event,
                                         gpointer         user_data)
 {
-	if (event->keyval == GDK_Page_Up || event->keyval == GDK_KP_Up || event->keyval == 'i')
+	if (event->keyval == GDK_Page_Up || event->keyval == GDK_KP_Up || event->keyval == GDK_i)
 		on_button4_clicked(NULL, NULL);
-	else if ((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK && event->keyval == 'p')
+	else if ((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK && event->keyval == GDK_p)
 		geo_photos_open_dialog_photo_correlate();
-	else if ((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK && event->keyval == 't')
+	else if ((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK && event->keyval == GDK_t)
 		tracks_open_tracks_dialog();
-	else if(event->keyval == GDK_Page_Down || event->keyval == 'o')
+	else if(event->keyval == GDK_Page_Down || event->keyval == GDK_o)
 		on_button5_clicked(NULL, NULL);
-	else if(event->keyval == 'm')
+	else if(event->keyval == GDK_m)
 		on_button76_clicked(NULL, NULL);
 	else if(event->keyval == GDK_space || event->keyval == GDK_F11)
 	{
@@ -3062,6 +3093,23 @@ on_itemgeocode1_activate               (GtkWidget       *widget,
 	return FALSE;
 }
 
+gboolean
+on_itemgeocode1_expose_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+	if (system ("command -v gpscorrelate > /dev/null") != 0) {
+		gtk_widget_set_sensitive (widget, FALSE);
+		gtk_widget_set_tooltip_text
+			(widget, _("gpscorrelate is necessary for this feature "
+                                   "but does not appear to be installed."));
+	} else {
+		gtk_widget_set_sensitive (widget, TRUE);
+		gtk_widget_set_tooltip_text (widget, NULL);
+	}
+
+	return FALSE;
+}
+
+
 
 void
 on_button39_clicked                    (GtkButton       *button,
@@ -3163,6 +3211,23 @@ on_eventbox3_button_release_event      (GtkWidget       *widget,
 	return FALSE;
 }
 
+int
+on_checkbutton14_expose_event (GtkWidget *togglebutton, gpointer user_data)
+{
+	if (system ("command -v jhead > /dev/null") != 0) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (togglebutton), FALSE);
+		gtk_widget_set_sensitive (togglebutton, FALSE);
+		gtk_widget_set_tooltip_text
+			(togglebutton, _("jhead is necessary for this feature "
+			                 "but does not appear to be installed."));
+	} else {
+		gtk_widget_set_sensitive (togglebutton, TRUE);
+		gtk_widget_set_tooltip_text (togglebutton, NULL);
+	}
+
+	return FALSE;
+}
+
 void
 on_checkbutton14_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
@@ -3304,6 +3369,72 @@ on_item17_button_release_event         (GtkWidget       *widget,
   return FALSE;
 }
 
+gboolean
+on_add_wp_button_release_event (GtkWidget *widget,
+                                GdkEventButton *event,
+                                gpointer user_data)
+{
+	float lat, lon;
+
+	lat = pixel2lat (global_zoom, global_y+mouse_y);
+	lon = pixel2lon (global_zoom, global_x+mouse_x);
+
+	append_waypoint_to_route (lat, lon);
+	repaint_all ();
+
+	return FALSE;
+}
+
+gboolean
+on_clear_route_button_release_event (GtkWidget *widget,
+                                     GdkEventButton *event,
+                                     gpointer user_data)
+{
+	reset_route ();
+	repaint_all ();
+	return FALSE;
+}
+
+gboolean
+on_save_route_gpx_button_release_event (GtkWidget *widget,
+                                        GdkEventButton *event,
+                                        gpointer user_data)
+{
+	char *filename;
+
+	filename = choose_save_file ("Unnamed.gpx");
+	save_route_as_gpx (filename);
+	g_free (filename);
+	return FALSE;
+}
+
+gboolean
+on_load_route_button_release_event (GtkWidget *widget,
+                                    GdkEventButton *event,
+                                    gpointer user_data)
+{
+	char *filename;
+
+	filename = choose_load_file ();
+	load_route (filename);
+	repaint_all ();
+	g_free (filename);
+	return FALSE;
+}
+
+gboolean
+on_save_route_tomtom_button_release_event (GtkWidget *widget,
+                                           GdkEventButton *event,
+                                           gpointer user_data)
+{
+	char *filename;
+
+	filename = choose_save_file ("Unnamed.itn");
+	save_route_as_tomtom_itn (filename);
+	g_free (filename);
+	return FALSE;
+}
+
 
 gboolean
 on_item18_button_release_event         (GtkWidget       *widget,
@@ -3737,13 +3868,28 @@ on_checkbutton16_toggled               (GtkToggleButton *togglebutton,
 	trip_livelog_on = gtk_toggle_button_get_active(togglebutton);
 }
 
+void
+tracklog_toolbutton_toggled (GtkToggleToolButton *button, gpointer data)
+{
+	GtkToggleButton *main_toggle =
+		GTK_TOGGLE_BUTTON (lookup_widget (window1, "checkbutton17"));
+
+	trip_logger_on = gtk_toggle_tool_button_get_active (button);
+
+	gtk_toggle_button_set_active (main_toggle, trip_logger_on);
+}
+
 void
 on_checkbutton17_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
+	GtkToggleToolButton *toolbar_toggle =
+		GTK_TOGGLE_TOOL_BUTTON (lookup_widget (window1, "button62"));
+
 	trip_logger_on = gtk_toggle_button_get_active(togglebutton);
 
-	
+	gtk_toggle_tool_button_set_active (toolbar_toggle, trip_logger_on);
+
 	if(trip_logger_on)
 	{
 		track_log_open();
@@ -3890,3 +4036,33 @@ toggle_tile_redownload (GtkToggleButton *togglebutton, gpointer data)
 	                       GCONF"/no_redownload",
 	                       global_no_redownload, NULL);
 }
+
+gboolean
+on_delete_waypoint_button_release_event (GtkWidget *widget,
+                                         GdkEventButton *event,
+                                         gpointer user_data)
+{
+	delete_waypoint_of_route (selected_wp);
+	selected_wp = NULL;
+	repaint_all ();
+
+	return FALSE;
+}
+
+gboolean
+on_insert_waypoint_button_release_event (GtkWidget *widget,
+                                         GdkEventButton *event,
+                                         gpointer user_data)
+{
+	insert_waypoint_before_of_route (selected_wp);
+	selected_wp = NULL;
+	repaint_all ();
+
+	return FALSE;
+}
+
+void
+on_help_menuitem_activate (GtkMenuItem *help_menuitem, gpointer user_data)
+{
+	system ("xdg-open " PACKAGE_HTML_DIR "/" PACKAGE ".html");
+}
