@@ -3,7 +3,7 @@
 #  include <config.h>
 #endif
 
-#define _XOPEN_SOURCE
+#define _XOPEN_SOURCE 700
 
 #include "globals.h"
 #include "geo_photos.h"
@@ -12,8 +12,8 @@
 #include "support.h"
 #include "interface.h"
 #include "exif.h"
-#include "tracks.h"
 #include "tile_management.h"
+#include "tracks.h"
 
 #include <glib.h>
 #include <stdio.h>
@@ -21,8 +21,15 @@
 #include <stdlib.h>
 #include <libexif/exif-data.h>
 #include <time.h>
+#include <math.h>
 
 #define PHOTO_DB "geophoto.db"
+#define PHOTO_DB_CREATE "CREATE TABLE photo " \
+	"(filename TEXT primary key, " \
+	"name TEXT, " \
+	"lat REAL, " \
+	"lon REAL, " \
+	"desc TEXT);"
 
 GList *geocode_photo_list;
 GtkWidget *dialog_image_data = NULL;
@@ -313,17 +320,10 @@ geo_photos_geocode_track_select_dialog (GtkButton       *button,
 	gtk_file_filter_add_pattern (filter, "*.GPX");
 	gtk_file_filter_set_name (filter, "GPX trace log files (*.gpx)");
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(widget), filter);
-	
-
-	filter = gtk_file_filter_new ();
-	gtk_file_filter_add_pattern (filter, "*.log");
-	gtk_file_filter_set_name (filter, PACKAGE_NAME " log files (*.log)");
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(widget), filter);
 
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER(widget), filter);
 
 
-	
 	if (gtk_dialog_run (GTK_DIALOG (widget)) == GTK_RESPONSE_ACCEPT)
 	{
 		char *filename;
@@ -697,11 +697,8 @@ geo_photo_set_add_to_database(GtkToggleButton *togglebutton)
 void
 geo_photo_close_dialog_photo_correlate()
 {
-	gchar *command_line;
 	GtkWidget *label;
 
-	prepare_perl();
-	
 	dialog_geocode_result = glade_xml_get_widget (gladexml,
 						      "dialog_geocode_result");
 
@@ -711,16 +708,7 @@ geo_photo_close_dialog_photo_correlate()
 	gtk_widget_show(dialog_geocode_result);
 	gtk_widget_hide(dialog_photo_correlate);
 
-	command_line = g_strdup_printf("/tmp/" PACKAGE "_geocode.pl '%s' '%s' '%d' '%d' '%d'", 
-					geocode_trackname, 
-					geocode_photodir,
-					geocode_timezone,
-					geocode_correction,
-					add_to_database);
-printf("commandline out thread: %s\n", command_line);	
-	
-	g_thread_create(geocode_thread, command_line, FALSE, NULL);
-	
+	g_thread_create(geocode_thread, NULL, FALSE, NULL);
 }
 
 
@@ -763,7 +751,68 @@ update_gps_time_label()
 
 
 
+static void
+exif2db (const char *db, char *photo)
+{
+	char *sql;
+	char *argv[] = {"jhead", photo, NULL};
+	char *line = NULL;
+	size_t linesz;
+	GPid jhead_pid;
+	int jhead_outfd;
+	FILE *jhead_stdout;
 
+	char lat_half=0, lon_half=0;
+	float lat_deg=NAN, lon_deg=NAN,
+	      lat_min=NAN, lon_min=NAN,
+	      lat_sec=NAN, lon_sec=NAN;
+
+	float lat, lon;
+
+	GError *error = NULL;
+
+	g_spawn_async_with_pipes (geocode_photodir, argv, NULL,
+	                          G_SPAWN_SEARCH_PATH, NULL, NULL,
+	                          &jhead_pid,
+	                          NULL, &jhead_outfd, NULL,
+	                          &error);
+	jhead_stdout = fdopen (jhead_outfd, "r");
+
+	while (getline (&line, &linesz, jhead_stdout) > -1)
+	{
+		/* At most one of these will actually do anything: */
+		sscanf (line, "GPS Latitude : %c %fd %fm %fs",
+		        &lat_half, &lat_deg, &lat_min, &lat_sec);
+		sscanf (line, "GPS Longitude: %c %fd %fm %fs",
+		        &lon_half, &lon_deg, &lon_min, &lon_sec);
+	}
+	free (line);
+
+	lat = lat_deg + lat_min/60 + lat_sec/3600;
+	if (lat_half == 'S') {
+		lat = -lat;
+	}
+	lon = lon_deg + lon_min/60 + lon_sec/3600;
+	if (lon_half == 'W') {
+		lon = -lon;
+	}
+
+	if (isnan(lat) || isnan(lon)) {
+		return;
+	}
+
+	char *photo_basename = g_path_get_basename (photo);
+	sql = sqlite3_mprintf ("INSERT INTO photo (filename, name, "
+	                                          "lat, lon, "
+	                                          "desc) "
+	                       "VALUES (%Q, %Q, %f, %f, %Q)",
+	                       photo, photo_basename,
+	                       lat, lon,
+	                       "unset");
+	g_free (photo_basename);
+	sql_execute (db, sql, NULL);
+	sqlite3_free (sql);
+}
 
 
 gpointer
@@ -774,22 +823,49 @@ geocode_thread(gpointer user_data)
 	gchar *standard_error;
 	gint exit_status;
 	GError *err = NULL;
-	char *command_line = (char *) user_data;
 	gboolean res;
 
-printf("commandline in thread: %s\n", command_line);	
-	
-	res = g_spawn_command_line_sync  (
-				command_line, 
-				&standard_output, 
-				&standard_error,
-				&exit_status,
-				&err);
+	char *timezone = g_strdup_printf ("%d", geocode_timezone);
+	char *correction = g_strdup_printf ("%d", geocode_correction);
+
+	GList *photos;
+
+	char **argv = g_new (char *, 9+g_list_length(geocode_photo_list)+1);
+	int argc = 0;
+	argv[argc++] = "gpscorrelate";
+	argv[argc++] = "-g";
+	argv[argc++] = geocode_trackname;
+	argv[argc++] = "-z";
+	argv[argc++] = timezone;
+	argv[argc++] = "--max-dist";
+	argv[argc++] = "120";
+	argv[argc++] = "--photooffset";
+	argv[argc++] = correction;
+
+	for (photos = geocode_photo_list;
+	     photos;
+	     photos = photos->next)
+	{
+		argv[argc++] = photos->data;
+	}
+
+	argv[argc] = NULL;
+
+	res = g_spawn_sync (NULL, argv, NULL,
+	                    G_SPAWN_SEARCH_PATH,
+	                    NULL, NULL,
+	                    &standard_output, 
+	                    &standard_error,
+	                    &exit_status,
+	                    &err);
+
+	g_free (timezone);
+	g_free (correction);
 	
 	if(!res)
 	{
 		fprintf (stderr, _("Error running \"%s\": %s\n"),
-		         command_line, err->message);
+		         "gpscorrelate", err->message);
 		g_error_free (err);
 	}
 	
@@ -802,356 +878,29 @@ printf("commandline in thread: %s\n", command_line);
 	
 	if(add_to_database)
 	{
+		GList *photos;
+		static char *db = NULL;
+
+		if (!db) {
+			db = g_build_filename (foxtrotgps_dir, PHOTO_DB, NULL);
+		}
+
+		sql_execute (db, PHOTO_DB_CREATE, NULL);
+
+		for (photos = geocode_photo_list;
+		     photos;
+		     photos = photos->next)
+		{
+
+			/* FIXME: catch/report exif2db errors? */
+			exif2db (db, (char *) photos->data);
+		}
+
 		item = lookup_widget(menu1, "item9");
-		gtk_menu_item_activate (GTK_MENU_ITEM(item));
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(item), TRUE);
 	}
 	
 	gdk_threads_leave();
-	
-	g_free(command_line);
 
 	return NULL;
 }
-
-
-void
-prepare_perl()
-{
-char *command;
-FILE *fp;
-int unused;
-
-{
-	printf("* %s\n", __PRETTY_FUNCTION__);
-
-	
-	
-	
-	
-	
-	
-	
-command = g_strdup(
-"#!/usr/bin/perl\n"
-"\n"
-"# (C)2008 Marcus Bauer, License: GPLv2, marus.bauer@gmail.com\n"
-"\n"
-"#\n"
-"# helper for FoxtrotGPS to geocode photos\n"
-"#\n"
-"\n"
-"# needs: gpscorrelate, sqlite3, jhead\n"
-"\n"
-"# input:\n"
-"# - .log or .gpx file\n"
-"# - photo list\n"
-"# - timezone\n"
-"# - correction\n"
-"# - do_insert2db\n"
-"\n"
-"# logic:\n"
-"# - convert to gpx\n"
-"# - correlate\n"
-"# - in2db foxtrotgps\n"
-"\n"
-"# gpscorrelate -g foo.gpx -z timezone --max-dist 120 --photooffset 5 p1.jpg ...\n"
-"\n"
-"$geocode_track = $ARGV[0];\n"
-"$geocode_dir   = $ARGV[1];\n"
-"$timezone      = $ARGV[2];\n"
-"$correction    = $ARGV[3];\n"
-"$do_insert2db  = $ARGV[4];\n"
-"\n"
-"$db = \"~/." PACKAGE "/geophoto.db\";\n"
-"\n"
-"#===================  MAIN  ====================\n"
-"\n"
-"print STDERR \"\n"
-"VARS: \n"
-"  geocode_track = $ARGV[0];\n"
-"  geocode_dir   = $ARGV[1];\n"
-"  timezone      = $ARGV[2];\n"
-"  correction    = $ARGV[3];\n"
-"  do_insert2db  = $ARGV[4];\n"
-"\";\n"
-"\n"
-"&test_for_helper_programs();\n"
-"&get_photos_from_dir();\n"
-"\n"
-"&convert2gpx();\n"
-"&correlate();\n"
-"&insert2db();\n"
-"\n"
-"#==================  END MAIN  ==================\n"
-"\n"
-"sub test_for_helper_programs()\n"
-"{\n"
-"	# jhead, sqlite3, gpscorrelate\n"
-"\n"
-"	$missing_prog = 0;\n"
-"\n"
-"	if(system(\"jhead -h >/dev/null\") == -1){ $install .= \" jhead,\";	$missing_prog = 1;};\n"
-"	if(system(\"sqlite3\") == -1)		 { $install .= \" sqlite3, \";	$missing_prog = 1;};\n"
-"	if(system(\"gpscorrelate -h >/dev/null\")  == -1)	 { $install .= \" gpscorrelate\"; 	$missing_prog = 1;};\n"
-"\n"
-"	$error_msg = 	\"Hello, I couldn\'t find the following helper programs: \\n\\n\" .\n"
-"			\"   $install \\n\\n\" .\n"
-"			\"-> You can find them in your Linux distribution.\\n\\n\" .\n"
-"			\"Please install them first and try again.\\n\";\n"
-"\n"
-"	if($missing_prog) {print STDOUT $error_msg;\n"
-"	exit;}\n"
-"}\n"
-"\n"
-"sub get_photos_from_dir()\n"
-"{\n"
-"	opendir(DIR, $geocode_dir) || die(\"Cannot open directory\");\n"
-"	@photos= readdir(DIR);\n"
-"	closedir(DIR);\n"
-"}\n"
-"\n"
-"sub convert2gpx\n"
-"{\n"
-"	# test if *.log \n"
-"	if($geocode_track =~ /\\.log$/)\n"
-"	{\n"
-"		&do_convert2gpx();\n"
-"	}\n"
-"}\n"
-"\n"
-"\n"
-"sub correlate\n"
-"{\n"
-"	#\n"
-"	# construct command line\n"
-"	#\n"
-"\n"
-"	chdir($geocode_dir);\n"
-"\n"
-"	foreach(@photos)\n"
-"	{\n"
-"		if(	-f \"$_\"   && \n"
-"			( ( $_ =~ /\\.jpg$/i ) ||\n"
-"			  ( $_ =~ /\\.jpeg$/i)\n"
-"			)\n"
-"		  )\n"
-"		{\n"
-"			$photo_list .= \" \'$_\'\";\n"
-"		}\n"
-"		else { print STDERR \"not a file: $_ \\n\"; }\n"
-"	}\n"
-"\n"
-"	# gpscorrelate ... \n"
-"	$command_line =  \"gpscorrelate \"\n"
-"			.\"-g \'$geocode_track\' \"\n"
-"			.\"-z $timezone \"\n"
-"			.\"--max-dist 120 \"\n"
-"			.\"--photooffset $correction\"\n"
-"			. $photo_list;\n"
-"\n"
-"	print STDERR \"$command_line\\n\";\n"
-"\n"
-"	$output = `$command_line`;\n"
-"\n"
-"	print STDOUT \"$output \\n\";\n"
-"\n"
-"}\n"
-"\n"
-"\n"
-"sub insert2db()\n"
-"{\n"
-"	if($do_insert2db)\n"
-"	{\n"
-"		&do_insert2db();\n"
-"	}\n"
-"	else\n"
-"	{\n"
-"		print STDERR \"Not adding to " PACKAGE_NAME " database\\n\";\n"
-"	}\n"
-"\n"
-"}\n"
-"\n"
-"\n"
-"#=========== END MAIN FUCTIONS, START HELPER FUNCTIONS ===========\n"
-"\n"
-"\n"
-"sub do_convert2gpx\n"
-"{\n"
-"	\n"
-"#---  open track logfile ---\n"
-"	\n"
-"open (INFILE, $geocode_track) or die $!;\n"
-"open (OUTFILE, \">/tmp/$$.gpx\") or die $!;\n"
-"\n"
-"\n"
-"	\n"
-"#--- start GPX ---\n"
-"\n"
-"$gpx = <<EOT\n"
-"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-"<gpx	version=\"1.0\"\n"
-"	creator=\"convert2gpx.pl http://www.foxtrotgps.org\"\n"
-"	xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
-"	xmlns=\"http://www.topografix.com/GPX/1/0\"\n"
-"	xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd\">\n"
-"\n"
-"<trk>\n"
-"<trkseg>\n"
-"EOT\n"
-";\n"
-"\n"
-"\n"
-"while (<INFILE>)\n"
-"{\n"
-"\n"
-"@arr = split(\',\',$_);\n"
-"chop @arr[6];\n"
-"\n"
-"$gpx .= <<EOT\n"
-"<trkpt lat=\"@arr[0]\" lon=\"@arr[1]\">\n"
-"  <ele>@arr[2]</ele>\n"
-"  <speed>@arr[3]</speed>\n"
-"  <course>@arr[4]</course>\n"
-"  <fix>3d</fix>\n"
-"  <hdop>@arr[5]</hdop>\n"
-"  <time>@arr[6]</time>\n"
-"</trkpt>\n"
-"EOT\n"
-";\n"
-"\n"
-"} #end while()\n"
-"\n"
-"\n"
-"$gpx .= <<EOT\n"
-"</trkseg>\n"
-"</trk>\n"
-"</gpx>\n"
-"EOT\n"
-";\n"
-"\n"
-"\n"
-"\n"
-"# write .gpx to /tmp/$$\n"
-"print OUTFILE $gpx;\n"
-"\n"
-"# set geocode_track to filename\n"
-"$geocode_track = \"/tmp/$$.gpx\";\n"
-"\n"
-"print STDERR \"track: $geocode_track\\n\"\n"
-"}\n"
-"\n"
-"\n"
-"#------------------------------------------------------------\n"
-"\n"
-"#---------------------------------------------\n"
-"# get exif info from photo and put into db\n"
-"#---------------------------------------------\n"
-"\n"
-"sub exif2db\n"
-"{\n"
-"\n"
-"local($photo) = @_;\n"
-"\n"
-"# ------------------\n"
-"# extract exif info\n"
-"# ------------------\n"
-"$lat_string = `jhead $photo | grep \"GPS Latitude\" `;\n"
-"$lon_string = `jhead $photo | grep \"GPS Longitude\" `;\n"
-"\n"
-"\n"
-"($lat_half, $lat_deg, $lat_min, $lat_sec) =\n"
-"	$lat_string =~ /^GPS Latitude : (N|S) +(\\d+)d +(\\d+)m +(\\d+\\.\\d+)s$/;\n"
-"\n"
-"($lon_half, $lon_deg, $lon_min, $lon_sec) =\n"
-"	$lon_string =~ /^GPS Longitude: (E|W) +(\\d+)d +(\\d+)m +(\\d+\\.\\d+)s$/;\n"
-"\n"
-"$lat = $lat_deg + $lat_min/60 + $lat_sec/3600;\n"
-"$lat = ($lat_half eq \"N\") ? $lat : -$lat;\n"
-"\n"
-"$lon = $lon_deg + $lon_min/60 + $lon_sec/3600;\n"
-"$lon = ($lon_half eq \"E\") ? $lon : -$lon;\n"
-"\n"
-"\n"
-"#----------------------------\n"
-"# extract filename from path \n"
-"#----------------------------\n"
-"$filename = \"\\\"\" . $photo . \"\\\"\";\n"
-"$file =  `basename $filename` ;\n"
-"chop $file;\n"
-"$file = \"\\\"\" . $file . \"\\\"\";\n"
-"$desc = \"\\\"unset\\\"\";\n"
-"\n"
-"\n"
-"\n"
-"print STDERR \"$file $lat - $lon\\n\";\n"
-"\n"
-"\n"
-"#-----------------------\n"
-"# insert into sqlite db\n"
-"#-----------------------\n"
-"\n"
-"if ($lat !=0 && $lon !=0)\n"
-"{\n"
-"	$res = `sqlite3 $db \'INSERT INTO photo VALUES (\"$geocode_dir/$photo\",$file,$lat,$lon,$desc)\'`;\n"
-"\n"
-"	print STDERR $res;\n"
-"}\n"
-"\n"
-"}\n"
-"\n"
-"\n"
-"sub do_insert2db \n"
-"{\n"
-"\n"
-"#-------------------------------------------\n"
-"# create database if it does not yet exist\n"
-"#-------------------------------------------\n"
-"$sql = \"\n"
-"CREATE TABLE \n"
-"	photo (\n"
-"		filename TEXT primary key, \n"
-"		name TEXT, \n"
-"		lat REAL, \n"
-"		lon REAL, \n"
-"		desc TEXT\n"
-"	)\n"
-"\";\n"
-"$res = `sqlite3 $db \'$sql\'`;\n"
-"\n"
-"print STDERR $res;\n"
-"\n"
-"\n"
-"#-----------------------\n"
-"# insert photos into db\n"
-"#----------------------\n"
-"foreach(@photos)\n"
-"{\n"
-"	#\n"
-"	# check if jpeg file\n"
-"	#\n"
-"	if(	-f \"$_\"   && \n"
-"		( ( $_ =~ /\\.jpg$/i ) ||\n"
-"		  ( $_ =~ /\\.jpeg$/i)\n"
-"		)\n"
-"	  )\n"
-"	{\n"
-"		&exif2db(\"$_\");\n"
-"	}\n"
-"}\n"
-"\n"
-"\n"
-"}\n"
-);
-}
-
-
-fp = fopen("/tmp/" PACKAGE "_geocode.pl", "w+");
-if (fp==0) 
-	printf("could not open tmp\n");	
-
-fprintf(fp, "%s", command);
-fclose(fp);
-
-unused = system ("chmod 700 /tmp/" PACKAGE "_geocode.pl");
-
-}
