@@ -20,12 +20,12 @@
 
 
 #include "globals.h"
-#include "tracks.h"
 #include "interface.h"
 #include "support.h"
 #include "converter.h"
 #include "map_management.h"
 #include "tile_management.h"
+#include "tracks.h"
 #include "util.h"
 
 GSList *loaded_track = NULL;
@@ -56,7 +56,7 @@ reset_loaded_track()
 }
 
 void
-print_track()
+paint_track()
 {	
 	GList *list;
 	int pixel_x, pixel_y, x,y, last_x = 0, last_y = 0;
@@ -122,30 +122,6 @@ print_track()
 			is_line = TRUE;
 		}
 	}
-	
-	
-	if(	
-		gpsdata && 7==6 &&
-		gpsdata->fix.longitude !=0 &&  
-		gpsdata->fix.latitude != 0 &&
-		last_x != 0 &&
-		last_y != 0
-		)
-	{
-		pixel_x = lon2pixel(global_zoom, deg2rad(gpsdata->fix.longitude));
-		pixel_y = lat2pixel(global_zoom, deg2rad(gpsdata->fix.latitude));
-			
-		x = pixel_x - global_x;
-		y = pixel_y - global_y;
-
-	
-		gdk_draw_line (pixmap, gc, x, y, last_x, last_y);
-		gtk_widget_queue_draw_area (
-			map_drawable, 
-			x-4, y-4,
-			8,8);		
-	}
-	
 }
 
 
@@ -223,7 +199,7 @@ track_log()
 	{
 		
 		time_sec = (time_t)gpsdata->fix.time;
-		ts = localtime(&time_sec);
+		ts = gmtime(&time_sec);
 		strftime(buffer, sizeof(buffer), "%Y-%m-%dT%H:%M:%SZ", ts);
 		
 		heartfreq = (hrmdata) ? hrmdata->freq : 0;
@@ -300,7 +276,7 @@ track_log_open()
 	
 	
 
-	strftime (buffer, sizeof (buffer), "%Y%m%d_%H%M%S.gpx", tm_struct);
+	strftime (buffer, sizeof (buffer), "%Y%m%d_%H%M%S%z.gpx", tm_struct);
 
 	
 	filename = g_strconcat(global_track_dir, buffer,NULL);
@@ -455,23 +431,41 @@ make_file_label(const char *file, char *full_file)
 	return label;
 }
 
+void
+show_bbox (bbox_t bbox) 
+{
+	GtkWidget *drawingarea, *range;
+	int track_zoom, width, height;
+
+	drawingarea = lookup_widget (window1, "drawingarea1");
+	width  = drawingarea->allocation.width;
+	height = drawingarea->allocation.height;
+
+	track_zoom = get_zoom_covering (width, height,
+	                                bbox.lat1, bbox.lon1,
+	                                bbox.lat2, bbox.lon2);
+	track_zoom = (track_zoom > 15) ? 15 : track_zoom;
+
+	set_mapcenter (rad2deg ((bbox.lat1+bbox.lat2)/2),
+	               rad2deg ((bbox.lon1+bbox.lon2)/2),
+	               track_zoom);
+
+	range = lookup_widget (window1, "vscale1");
+	gtk_range_set_value (GTK_RANGE (range), (double) global_zoom);
+}
+
 gboolean
 tracks_on_file_button_release_event   (	GtkWidget       *widget,
                                         GdkEventButton  *event,
                                         gpointer         user_data)
 {
-	GtkWidget *drawingarea, *range, *widget_tmp;
-	int track_zoom, width, height;
+	GtkWidget *widget_tmp;
 	char *file;
 	bbox_t bbox;
 	
 	widget_tmp = lookup_widget(window1, "button79");
 	gtk_widget_show(widget_tmp);
 	
-	drawingarea = lookup_widget(window1, "drawingarea1");
-	width  = drawingarea->allocation.width;
-	height = drawingarea->allocation.height;
-	
 	file = (char *) user_data;
 
 	if (widget && window12) {
@@ -512,28 +506,15 @@ tracks_on_file_button_release_event   (	GtkWidget       *widget,
 	}
 	else
 		loaded_track = load_log_file_into_list(file);
-	
-	
-	
-	
-	
-	
-	bbox = get_track_bbox(loaded_track);
-	
-	track_zoom = get_zoom_covering(width, height, bbox.lat1, bbox.lon1, bbox.lat2, bbox.lon2);
-	track_zoom = (track_zoom > 15) ? 15 : track_zoom;
-	
-	
-	if(loaded_track)
-		set_mapcenter(rad2deg((bbox.lat1+bbox.lat2)/2), rad2deg((bbox.lon1+bbox.lon2)/2), track_zoom);
 
-	
-	range = lookup_widget(window1, "vscale1");
-	gtk_range_set_value(GTK_RANGE(range), (double) global_zoom);
+	if(loaded_track)
+	{
+		bbox = get_track_bbox (loaded_track);
+		show_bbox (bbox);
+	}
 
 	paint_loaded_track();
-	
-	
+
 	return FALSE;	
 }
 
@@ -576,6 +557,11 @@ load_log_file_into_list(char *file)
 	FILE *fd;
 
 	fd = fopen(file, "r");
+
+	if (!fd) {
+		return list;
+	}
+
 	while(fgets(line,120, fd))
 	{
 		trackpoint_t *tp;
@@ -595,7 +581,9 @@ load_log_file_into_list(char *file)
 
 		list = g_slist_append(list, tp);
 	}
-	
+
+	fclose (fd);
+
 	return list;
 }
 
@@ -747,12 +735,12 @@ parse_ols_XML_nodes(xmlNode *node)
 								if (lonlat[1])
 								{
 									lat = atof(lonlat[1]);
+									tp->lat = deg2rad(lat);
+									tp->lon = deg2rad(lon);
+									list = g_slist_append(list, tp);
 								}
 							}
-							tp->lat = deg2rad(lat);
-							tp->lon = deg2rad(lon);
 							g_strfreev (lonlat);
-							list = g_slist_append(list, tp);
 						}
 						inner_cur_node = inner_cur_node->next;
 					}
@@ -776,10 +764,11 @@ fetch_track(GtkWidget *widget, char *service, char *start, char *end)
 		fetch_openrouteservice_track(widget, start, end);
 	else
 	{
-		char *err_msg = g_strdup (_("<span color='#aa0000'><b>"
-		                            "Unknown service selected"
-		                            "</b></span>\n"
-		                            "This is a bug in FoxtrotGPS!"));
+		char *err_msg = g_strdup_printf (_("<span color='#aa0000'><b>"
+		                                   "Unknown service selected"
+		                                   "</b></span>\n"
+		                                   "This is a bug in %s!"),
+		                                 _(PACKAGE_NAME));
 		GtkWidget *tmpWidget;
 
 		tmpWidget = lookup_widget(dialog10, "label190");
@@ -821,7 +810,6 @@ void fetch_openrouteservice_track(GtkWidget *widget, char *start, char *end)
 	char *startlonstr;
 	char *endlatstr;
 	char *endlonstr;
-	char *myEnd = g_strdup(end);
 	char *parseptr;
 	startlatstr = strtok_r (start, ",", &parseptr);
 	startlonstr = strtok_r (NULL, ",", &parseptr);
@@ -880,14 +868,8 @@ void fetch_openrouteservice_track(GtkWidget *widget, char *start, char *end)
 
 void process_fetched_track(postreply_t *reply, bool save_gpx)
 {
-	GtkWidget *range, *drawingarea;
-	int track_zoom, width, height;
 	bbox_t bbox;
 	
-	drawingarea = lookup_widget(window1, "drawingarea1");
-	width  = drawingarea->allocation.width;
-	height = drawingarea->allocation.height;
-	
 	if(loaded_track)
 	{
 		if (save_gpx)
@@ -900,7 +882,7 @@ void process_fetched_track(postreply_t *reply, bool save_gpx)
 		
 			time_epoch_sec = time(NULL);
 			tm_struct = localtime(&time_epoch_sec);
-			strftime(buffer, sizeof(buffer), "nav%Y%m%d_%H%M%S.gpx", tm_struct);
+			strftime(buffer, sizeof(buffer), "nav%Y%m%d_%H%M%S%z.gpx", tm_struct);
 		
 			filename = g_strconcat(global_track_dir, buffer,NULL);
 	
@@ -919,24 +901,16 @@ void process_fetched_track(postreply_t *reply, bool save_gpx)
 			g_free(filename);
 		}
 		
-		bbox = get_track_bbox(loaded_track);
-		
-		track_zoom = get_zoom_covering(width, height, bbox.lat1, bbox.lon1, bbox.lat2, bbox.lon2);
-		track_zoom = (track_zoom > 15) ? 15 : track_zoom;
-	
 		gdk_threads_enter();
 		{
-			
 			if(loaded_track)
-				set_mapcenter(rad2deg((bbox.lat1+bbox.lat2)/2), rad2deg((bbox.lon1+bbox.lon2)/2), track_zoom);
-		
+			{
+				bbox = get_track_bbox (loaded_track);
+				show_bbox (bbox);
+			}
+
 			paint_loaded_track();
 			gtk_widget_hide(dialog10);
-			
-			
-			
-			range = lookup_widget(window1, "vscale1");
-			gtk_range_set_value(GTK_RANGE(range), (double) global_zoom);
 		}
 		gdk_threads_leave();
 	}
